@@ -258,7 +258,7 @@ QList<DataType> SchemaResolver::getTableColumnDataTypes(const QString& database,
 StrHash<QStringList> SchemaResolver::getAllTableColumns(const QString &database)
 {
     StrHash< QStringList> tableColumns;
-    for (QString table : getTables(database))
+    for (const QString& table : getTables(database))
         tableColumns[table] = getTableColumns(database, table);
 
     return tableColumns;
@@ -518,10 +518,15 @@ QStringList SchemaResolver::getColumnsFromDdlUsingPragma(const QString& ddl)
     return QStringList();
 }
 
-QStringList SchemaResolver::getColumnsUsingPragma(const QString& tableOrView)
+QStringList SchemaResolver::getColumnsUsingPragma(const QString& tableOrView, bool onlyReal)
 {
-    static_qstring(query, "PRAGMA table_info(%1)");
-    SqlQueryPtr results = db->exec(query.arg(wrapObjIfNeeded(tableOrView)));
+    return getColumnsUsingPragma("main", tableOrView, onlyReal);
+}
+
+QStringList SchemaResolver::getColumnsUsingPragma(const QString& database, const QString& tableOrView, bool onlyReal)
+{
+    static_qstring(query, "PRAGMA %1.%3(%2)");
+    SqlQueryPtr results = db->exec(query.arg(wrapObjIfNeeded(database), wrapObjIfNeeded(tableOrView), onlyReal ? "table_info" : "table_xinfo"));
     if (results->isError())
     {
         qWarning() << "Could not get column list using PRAGMA for table or view:" << tableOrView << ", error was:" << results->getErrorText();
@@ -957,16 +962,15 @@ SchemaResolver::ObjectType SchemaResolver::objectTypeFromQueryType(const SqliteQ
 
 QStringList SchemaResolver::getIndexesForTable(const QString& database, const QString& table)
 {
-    static_qstring(idxForTableTpl, "SELECT name FROM %1.sqlite_master WHERE type = 'index' AND (tbl_name = '%2' OR lower(tbl_name) = lower('%2'));");
+    static_qstring(idxForTableTpl, "PRAGMA %1.index_list(%2);");
 
     QString query = idxForTableTpl.arg(wrapObjName(database), wrapObjIfNeeded(table));
     SqlQueryPtr results = db->exec(query, dbFlags);
 
     QStringList indexes;
-    QString value;
-    for (SqlResultsRowPtr row : results->getAll())
+    for (SqlResultsRowPtr& row : results->getAll())
     {
-        value = row->value(0).toString();
+        QString value = row->value("name").toString();
         if (isFilteredOut(value, "index"))
             continue;
 
@@ -989,7 +993,7 @@ QStringList SchemaResolver::getTriggersForTable(const QString& database, const Q
     SqlQueryPtr results = db->exec(query, dbFlags);
 
     QStringList names;
-    for (SqlResultsRowPtr row : results->getAll())
+    for (SqlResultsRowPtr& row : results->getAll())
         names << row->value(0).toString();
 
     return names;
@@ -1002,11 +1006,8 @@ QStringList SchemaResolver::getTriggersForTable(const QString& table)
 
 QStringList SchemaResolver::getTriggersForView(const QString& database, const QString& view)
 {
-    QStringList names;
-    for (SqliteCreateTriggerPtr trig : getParsedTriggersForView(database, view))
-        names << trig->trigger;
-
-    return names;
+    // Currently getting triggers for a view is same as it is for a table, but it may differ in future.
+    return getTriggersForTable(database, view);
 }
 
 QStringList SchemaResolver::getTriggersForView(const QString& view)
